@@ -25,11 +25,10 @@ typedef struct {
 typedef struct {
   cmark_chunk info;
   cmark_chunk literal;
-  int fence_length;
-  /* fence_offset must be 0-3, so we can use int8_t */
-  int8_t fence_offset;
+  uint8_t fence_length;
+  uint8_t fence_offset;
   unsigned char fence_char;
-  bool fenced;
+  int8_t fenced;
 } cmark_code;
 
 typedef struct {
@@ -47,7 +46,14 @@ typedef struct {
   cmark_chunk on_exit;
 } cmark_custom;
 
+enum cmark_node__internal_flags {
+  CMARK_NODE__OPEN = (1 << 0),
+  CMARK_NODE__LAST_LINE_BLANK = (1 << 1),
+};
+
 struct cmark_node {
+  cmark_strbuf content;
+
   struct cmark_node *next;
   struct cmark_node *prev;
   struct cmark_node *parent;
@@ -60,13 +66,8 @@ struct cmark_node {
   int start_column;
   int end_line;
   int end_column;
-
-  cmark_node_type type;
-
-  bool open;
-  bool last_line_blank;
-
-  cmark_strbuf string_content;
+  uint16_t type;
+  uint16_t flags;
 
   union {
     cmark_chunk literal;
@@ -79,6 +80,9 @@ struct cmark_node {
   } as;
 };
 
+static CMARK_INLINE cmark_mem *cmark_node_mem(cmark_node *node) {
+  return node->content.mem;
+}
 CMARK_EXPORT int cmark_node_check(cmark_node *node, FILE *out);
 
 #ifdef __cplusplus
